package com.xiaoliu;

import com.xiaoliu.codec.DecodeHandler;
import com.xiaoliu.codec.EncodeHandler;
import com.xiaoliu.handler.FilePacketClientHandler;
import com.xiaoliu.handler.FileSendClientHandler;
import com.xiaoliu.handler.LoginResponseHandler;
import com.xiaoliu.protocol.FilePacket;
import com.xiaoliu.protocol.request.LoginPacket;
import io.netty.bootstrap.Bootstrap;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioSocketChannel;
import io.netty.handler.stream.ChunkedWriteHandler;
import lombok.extern.slf4j.Slf4j;

import java.io.File;
import java.util.concurrent.TimeUnit;

@Slf4j
public class Client {

    private static final String HOST = System.getProperty("host", "127.0.0.1");

    private static final int PORT = Integer.parseInt(System.getProperty("port", "8080"));

    private static ChannelFuture future;

    private static NioEventLoopGroup group;

    public static void init() throws Exception{
        init(HOST, PORT);
    }

    public static void init(String host, int port) throws Exception{

        if(host == null || host.isEmpty()){
            host = HOST;
        }

        if(port == 0){
            port = PORT;
        }

        log.info("客户端启动开始...");

        Bootstrap bootstrap = new Bootstrap();

        group = new NioEventLoopGroup(2);

        bootstrap.group(group)
                .channel(NioSocketChannel.class)
                .option(ChannelOption.CONNECT_TIMEOUT_MILLIS, 5000)
                .option(ChannelOption.SO_KEEPALIVE, true)
                .option(ChannelOption.TCP_NODELAY, true)
                .handler(new ChannelInitializer<NioSocketChannel>() {
                    @Override
                    protected void initChannel(NioSocketChannel channel) throws Exception {
                        ChannelPipeline pipeline = channel.pipeline();
                        //pipeline.addLast(new FileReceiveClientHandler());
                        pipeline.addLast(new FileSendClientHandler());
                        pipeline.addLast(new DecodeHandler());
                        pipeline.addLast(new EncodeHandler());
                        pipeline.addLast(new ChunkedWriteHandler());
                        pipeline.addLast(new LoginResponseHandler());
                        pipeline.addLast(new FilePacketClientHandler());
                        // pipeline.addLast(new MyClientHandler());
                    }
                });

        future = bootstrap.connect(host, port).sync();
        if (future.isSuccess()) {
            log.info("连接服务器成功");
            Channel channel = future.channel();
            joinCluster(channel);

        } else {
            log.info("连接服务器失败");
        }

        future.channel().closeFuture().sync();
    }

    private static void joinCluster(Channel channel){
        LoginPacket loginPacket = new LoginPacket("node1");
        channel.writeAndFlush(loginPacket);
        try {
            Thread.sleep(2000);
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }
    }

    public static ChannelFuture getFuture() {
        if(future == null){
            throw new RuntimeException("请先使用init方法，连接到服务器...");
        }
        return future;
    }

    public static void send(File file){
        ChannelFuture future1 = getFuture();
        send(future1.channel(), file);
    }

    public static void send(Channel channel, File file) {
        FilePacket filePacket = new FilePacket(file);
        channel.writeAndFlush(filePacket);
    }

    public static void shutdown(){
        if(future != null){
            future.channel().close();
        }
        if(group != null){
            group.shutdownGracefully(10, 30, TimeUnit.SECONDS);
        }
    }
}

