package com.xiaoliu.window;

import com.xiaoliu.Client;
import lombok.extern.slf4j.Slf4j;
import com.formdev.flatlaf.FlatDarculaLaf;

import javax.swing.*;
import java.awt.*;
import java.io.File;

@Slf4j
public class MainWindow extends JFrame {

    private static JTextArea logTextArea;

    private static JLabel statusLabel;

    public MainWindow() {
        // 设置窗口标题
        setTitle("影像Netty客户端");

        // 设置默认的关闭操作
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        setVisible(true);

        // 设置窗口大小
        setSize(500, 400);

        // 居中显示窗口
        setLocationRelativeTo(null);

        //初始化状态组件
        initStatusLabel();

        // 初始化UI组件
        initializeComponents();
    }

    private void initStatusLabel() {
        statusLabel = new JLabel();
        statusLabel.setForeground(Color.RED);
        statusLabel.setText("未连接");
    }

    public void setStatusLabel(String text, Color color ){
        if(statusLabel == null){
            initStatusLabel();
        }
        statusLabel.setText(text);
        statusLabel.setForeground(color);
    }

    private void initializeComponents() {
        // 在这里添加你的组件初始化代码
        // 创建一个JTextArea用于显示日志
        logTextArea = new JTextArea();
        logTextArea.setEditable(false); // 设置为不可编辑
        logTextArea.setEnabled(false);

        // 将JTextArea放入JScrollPane中以支持滚动
        JScrollPane scrollPane = new JScrollPane(logTextArea);

        // 将JScrollPane添加到窗口中
        getContentPane().add(scrollPane, BorderLayout.CENTER);
        JLabel adress  = new JLabel();
        adress.setText("地址：");
        JTextField textField = new JTextField(20); // 参数20指定了文本框的列数
        JButton sendButton = new JButton("连接");
        JButton stopButton = new JButton("断开");
        stopButton.setEnabled(false);
        sendButton.addActionListener(e ->{
            String text = textField.getText();
            String[] split = text.split(":");
            if(split.length < 2){
                JOptionPane.showMessageDialog(this, "请输入正确地址", "输入错误", JOptionPane.ERROR_MESSAGE);
                return;
            }
            String host = split[0];
            String port = split[1];
            setLogText("连接到: "+ host + ":" + port);
            new Thread(() -> {
                //新起一个县城去初始化netty
                try {
                    sendButton.setEnabled(false);
                    setLogText("连接成功...");
                    setStatusLabel("已连接", Color.GREEN);
                    stopButton.setEnabled(true);
                    Client.init(host, Integer.parseInt(port));
                } catch (Exception ex) {
                    setLogText("连接失败...");
                    stopButton.setEnabled(false);
                    setStatusLabel("已连接", Color.RED);
                    throw new RuntimeException(ex);
                }
            }).start();
        });

        JPanel northPanel = new JPanel();
        northPanel.add(adress);
        northPanel.add(textField);
        northPanel.add(sendButton);
        northPanel.add(statusLabel, BorderLayout.WEST);

        getContentPane().add(northPanel, BorderLayout.NORTH);

        JPanel bottomPanel = new JPanel();

        // 示例：添加一个按钮，点击时向文本区域添加日志信息
        JButton startButton = new JButton("发送");


        startButton.addActionListener(e -> {
            JFileChooser fileChooser = new JFileChooser();
            fileChooser.setCurrentDirectory(new java.io.File(System.getProperty("user.home") + "/Desktop"));

            // 显示文件选择对话框，返回值为用户操作的结果
            int result = fileChooser.showOpenDialog(this);

            // 如果用户点击了"打开"按钮
            if (result == JFileChooser.APPROVE_OPTION) {
                // 获取选中的文件路径
                String selectedFilePath = fileChooser.getSelectedFile().getPath();
                File file = new File(selectedFilePath);
                log.info("文件存在吗：{}", file.exists());
                log.info("文件大小：{}", file.length());
                new Thread(() -> {
                    //开一个新县城去泡发送程序
                    setLogText("文件发送开始： "+selectedFilePath);
                    Client.send(file);
                }).start();

            }
        });

        stopButton.addActionListener(e -> {
            setLogText("连接中止...");
            Client.shutdown();
            setStatusLabel("未连接", Color.RED);
            sendButton.setEnabled(true);
        });


        bottomPanel.add(startButton, BorderLayout.CENTER);
        bottomPanel.add(stopButton, BorderLayout.CENTER);
        getContentPane().add(bottomPanel, BorderLayout.SOUTH);

    }

    public static void main(String[] args) {
        log.info("服务启动中...");
        try {
            UIManager.setLookAndFeel(new FlatDarculaLaf());
        } catch (UnsupportedLookAndFeelException e) {
            System.err.println("UnsupportedLookAndFeelException: " + e.getMessage());
        }
        SwingUtilities.invokeLater(() -> new MainWindow().setVisible(true));
    }

    public JTextArea getLogTextArea(){
        if(logTextArea == null){
            initLogTextArea();
        }
        return logTextArea;
    }

    public void setLogText(String text){
        logTextArea.append(text + "\n");
    }

    public void initLogTextArea(){
        // 在这里添加你的组件初始化代码
        // 创建一个JTextArea用于显示日志
        logTextArea = new JTextArea();
        logTextArea.setEditable(false); // 设置为不可编辑
        logTextArea.setEnabled(false);

        // 将JTextArea放入JScrollPane中以支持滚动
        JScrollPane scrollPane = new JScrollPane(logTextArea);

        // 将JScrollPane添加到窗口中
        getContentPane().add(scrollPane, BorderLayout.CENTER);
    }

}
