package com.xiaoliu.util;

import com.xiaoliu.protocol.attribute.Attributes;
import com.xiaoliu.protocol.request.LoginPacket;
import com.xiaoliu.protocol.session.Session;
import io.netty.channel.Channel;
import lombok.extern.slf4j.Slf4j;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

@Slf4j
public class LoginUtil {

	private static final Map<String, Channel> LOGIN_MAP = new HashMap<>();

	public static void bindUser(LoginPacket login, Channel channel) {
		LOGIN_MAP.put(login.getId(), channel);
		channel.attr(Attributes.userAttr).set(login);
	}

	public static void unBind(Channel channel) {
		if (hasLogin(channel)) {
			LoginPacket login = getLogin(channel);
			LOGIN_MAP.remove(login.getId());
			channel.attr(Attributes.userAttr).set(null);
            log.info("{} {}退出集群", new Date(), login.getId());
		}
	}

	private static boolean hasLogin(Channel channel) {
		return channel.hasAttr(Attributes.userAttr);
	}

	private static LoginPacket getLogin(Channel channel) {
		return channel.attr(Attributes.userAttr).get();
	}

	public static Map<String, Channel> getLoginMap() {
		return LOGIN_MAP;
	}
}
