package com.xiaoliu.util;

import com.xiaoliu.protocol.attribute.Attributes;
import com.xiaoliu.protocol.session.Session;
import io.netty.channel.Channel;
import lombok.extern.slf4j.Slf4j;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

@Slf4j
public class SessionUtil {

	private static final Map<String, Channel> NODE_ID_CHANNEL_MAP = new HashMap<>();

	public static void bindSession(Session session, Channel channel) {
		NODE_ID_CHANNEL_MAP.put(session.getNodeId(), channel);
		channel.attr(Attributes.SESSION).set(session);
	}

	public static void unBindSession(Channel channel) {
		if (hasLogin(channel)) {
			Session session = getSession(channel);
			NODE_ID_CHANNEL_MAP.remove(session.getNodeId());
			channel.attr(Attributes.SESSION).set(null);
            log.info("{} {}退出集群", new Date(), session);
		}
	}

	private static boolean hasLogin(Channel channel) {
		return channel.hasAttr(Attributes.SESSION);
	}

	private static Session getSession(Channel channel) {
		return channel.attr(Attributes.SESSION).get();
	}

	public static Map getNodeIdChannelMap() {
		return NODE_ID_CHANNEL_MAP;
	}
}
